@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2010 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -18,9 +18,6 @@
  */
 
 # include  "vvp_priv.h"
-#ifdef HAVE_MALLOC_H
-# include  <malloc.h>
-#endif
 # include  <stdlib.h>
 # include  <math.h>
 # include  <string.h>
@@ -96,7 +93,8 @@ static char* draw_C4_to_string(ivl_net_const_t cptr)
       for (idx = 0 ;  idx < ivl_const_width(cptr) ;  idx += 1) {
 	    char bitchar = bits[ivl_const_width(cptr)-idx-1];
 	    *dp++ = bitchar;
-	    assert((dp - result) < result_len);
+	    assert(dp >= result);
+	    assert((unsigned)(dp - result) < result_len);
       }
 
       strcpy(dp, ">");
@@ -146,7 +144,8 @@ static char* draw_C8_to_string(ivl_net_const_t cptr,
 		  assert(0);
 		  break;
 	    }
-	    assert(dp - result < nresult);
+	    assert(dp >= result);
+	    assert((unsigned)(dp - result) < nresult);
       }
 
       strcpy(dp, ">");
@@ -260,7 +259,8 @@ static char* draw_net_input_drive(ivl_nexus_t nex, ivl_nexus_ptr_t nptr)
 		  dp += ivl_logic_width(lptr);
 		  *dp++ = '>';
 		  *dp = 0;
-		  assert((dp-result) <= result_len);
+		  assert(dp >= result);
+		  assert((unsigned)(dp - result) <= result_len);
 		  return result;
 	    } else {
 		  char val[4];
@@ -279,7 +279,8 @@ static char* draw_net_input_drive(ivl_nexus_t nex, ivl_nexus_ptr_t nptr)
 		  dp += 3*ivl_logic_width(lptr);
 		  *dp++ = '>';
 		  *dp = 0;
-		  assert((dp-result) <= result_len);
+		  assert(dp >= result);
+		  assert((unsigned)(dp - result) <= result_len);
 		  return result;
 	    }
       }
@@ -295,7 +296,8 @@ static char* draw_net_input_drive(ivl_nexus_t nex, ivl_nexus_ptr_t nptr)
 		  dp += ivl_logic_width(lptr);
 		  *dp++ = '>';
 		  *dp = 0;
-		  assert((dp-result) <= result_len);
+		  assert(dp >= result);
+		  assert((unsigned)(dp - result) <= result_len);
 		  return result;
 	    } else {
 		  char val[4];
@@ -314,7 +316,8 @@ static char* draw_net_input_drive(ivl_nexus_t nex, ivl_nexus_ptr_t nptr)
 		  dp += 3*ivl_logic_width(lptr);
 		  *dp++ = '>';
 		  *dp = 0;
-		  assert((dp-result) <= result_len);
+		  assert(dp >= result);
+		  assert((unsigned)(dp - result) <= result_len);
 		  return result;
 	    }
       }
@@ -474,7 +477,7 @@ static char* draw_net_input_drive(ivl_nexus_t nex, ivl_nexus_ptr_t nptr)
 
       }
 
-      fprintf(stderr, "internal error: no input to nexus.\n");
+      fprintf(stderr, "vvp.tgt error: no input to nexus.\n");
       assert(0);
       return strdup("C<z>");
 }
