@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2010 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2011 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -561,7 +561,7 @@ static int show_stmt_assign_sig_real(ivl_statement_t net)
 
 	// For now, only support 1-dimensional arrays.
       assert(ivl_signal_dimensions(var) == 1);
- 
+
 	// Calculate the word index into an index register
       ivl_expr_t word_ex = ivl_lval_idx(lval);
       int word_ix = allocate_word();
@@ -630,8 +630,8 @@ static int show_stmt_assign_nb_real(ivl_statement_t net)
 	    /* This method no longer works since variable arrays do not
 	     * support <variable_id>_<idx> access any more. We need real
 	     * array specific opcodes (%assign/ar, etc.). */
-	    fprintf(stderr, "%s:%u: vvp-tgt sorry: non-blocking assignment "
-	            "to a real array word is not supported.\n",
+	    fprintf(stderr, "%s:%u: tgt-vvp sorry: non-blocking assignment "
+	            "to a real array word is not supported in V0.9.\n",
 	            ivl_expr_file(rval), ivl_expr_lineno(rval));
 	    exit(1);
       }
@@ -723,12 +723,12 @@ static int show_stmt_assign_nb(ivl_statement_t net)
 
 	    if (cnt) {
 		  int count_index = allocate_word();
-		  char*type = ivl_expr_signed(cnt) ? "/s" : "";
+		  const char*type = ivl_expr_signed(cnt) ? "/s" : "";
 		  draw_eval_expr_into_integer(cnt, count_index);
 		  fprintf(vvp_out, "    %%evctl%s %s, %d;\n", type, name,
 		                   count_index);
 		  clr_word(count_index);
-	    } else { 
+	    } else {
 		  fprintf(vvp_out, "    %%evctl/i %s, %lu;\n", name, count);
 	    }
       } else {
@@ -1201,10 +1201,33 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
       }
       if (ivl_signal_width(lsig) > ivl_signal_width(rsig) ||
           (part_off_ex && get_number_immediate(part_off_ex) != 0)) {
-	    fprintf(stderr, "%s:%u: vvp-tgt sorry: cannot %s signal to "
-	            "a bit/part select.\n", ivl_expr_file(rval),
-	            ivl_expr_lineno(rval), command_name);
-	    exit(1);
+	      /* Normalize the bit/part select. */
+	    long real_msb = ivl_signal_msb(lsig);
+	    long real_lsb = ivl_signal_lsb(lsig);
+	    long use_wid = ivl_signal_width(rsig);
+	    long use_lsb, use_msb;
+	    if (real_msb >= real_lsb) {
+		  use_lsb = get_number_immediate(part_off_ex);
+		  use_lsb += real_lsb;
+		  use_msb = use_lsb + use_wid - 1;
+	    } else {
+		  use_lsb = real_lsb;
+		  use_lsb -= get_number_immediate(part_off_ex);
+		  use_msb = use_lsb;
+		  use_msb -= use_wid - 1;
+	    }
+	      /* This is technically the wrong file/line object. It should
+	       * be the statement. */
+	    fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s signal to a ",
+	            ivl_expr_file(rval), ivl_expr_lineno(rval), command_name);
+	    if (use_wid > 1) {
+		  fprintf(stderr, "part select (%s[%lu:%lu]).\n",
+		          ivl_signal_basename(lsig), use_msb, use_lsb);
+	    } else {
+		  fprintf(stderr, "bit select (%s[%lu]).\n",
+		          ivl_signal_basename(lsig), use_lsb);
+	    }
+	    vvp_errors += 1;
       }
 
 	/* At least for now, only handle force to fixed words of an array. */
@@ -1212,17 +1235,44 @@ static void force_link_rval(ivl_statement_t net, ivl_expr_t rval)
 	    assert(number_is_immediate(lword_idx, IMM_WID, 0));
 	    assert(! number_is_unknown(lword_idx));
 	    use_lword = get_number_immediate(lword_idx);
+	      /* We do not currently support using a word from a variable
+	       * array as the L-value (Icarus extension). */
+	    if (ivl_signal_type(lsig) == IVL_SIT_REG) {
+		    /* Normalize the array access. */
+		  long real_word = use_lword;
+		  real_word += ivl_signal_array_base(lsig);
+		    /* This is technically the wrong file/line object. It
+		     * should be the statement. */
+		  fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s to the "
+		          "word of a variable array (%s[%ld]).\n",
+		          ivl_expr_file(rval), ivl_expr_lineno(rval),
+		          command_name, ivl_signal_basename(lsig), real_word);
+		  vvp_errors += 1;
+	    }
       }
 
       if ((rword_idx = ivl_expr_oper1(rval)) != 0) {
+	    assert(ivl_signal_dimensions(rsig) != 0);
 	    assert(number_is_immediate(rword_idx, IMM_WID, 0));
 	    assert(! number_is_unknown(rword_idx));
 	    use_rword = get_number_immediate(rword_idx);
+	      /* We do not currently support using a word from a variable
+	       * array as the R-value. */
+	    if (ivl_signal_type(rsig) == IVL_SIT_REG) {
+		    /* Normalize the array access. */
+		  long real_word = use_rword;
+		  real_word += ivl_signal_array_base(rsig);
+		  fprintf(stderr, "%s:%u: tgt-vvp sorry: cannot %s from the "
+		          "word of a variable array (%s[%ld]).\n",
+		          ivl_expr_file(rval), ivl_expr_lineno(rval),
+		          command_name, ivl_signal_basename(rsig), real_word);
+		  vvp_errors += 1;
+	    }
+      } else {
+	    assert(ivl_signal_dimensions(rsig) == 0);
+	    use_rword = 0;
       }
 
-      assert(ivl_signal_dimensions(rsig) == 0);
-      use_rword = 0;
-
       fprintf(vvp_out, "    %s/link", command_name);
       fprintf(vvp_out, " v%p_%lu", lsig, use_lword);
       fprintf(vvp_out, ", v%p_%lu;\n", rsig, use_rword);
@@ -1634,7 +1684,7 @@ static int show_stmt_repeat(ivl_statement_t net, ivl_scope_t sscope)
       unsigned lab_top = local_count++, lab_out = local_count++;
       ivl_expr_t expr = ivl_stmt_cond_expr(net);
       struct vector_info cnt = draw_eval_expr(expr, 0);
-      char *sign = ivl_expr_signed(expr) ? "s" : "u";
+      const char *sign = ivl_expr_signed(expr) ? "s" : "u";
 
 	/* Test that 0 < expr */
       fprintf(vvp_out, "T_%u.%u %%cmp/%s 0, %u, %u;\n", thread_count,
