@@ -335,7 +335,7 @@ char* draw_Cr_to_string(double value)
       }
 
       sign = 0;
-      if (value < 0) {
+      if (value < 0.0 || (value == 0.0 && 1.0/value < 0.0)) {
 	    sign = 0x4000;
 	    value *= -1;
       }
