@@ -728,7 +728,7 @@ if((lt)&&(lt->numfacs))
 		lt->numfacs = facs_encountered;				/* don't process alias value changes ever */
 		}
 
-	if(aliascache) free(aliascache);
+	free(aliascache);
 	}
 }
 
@@ -1144,7 +1144,7 @@ unsigned int total_chgs;
 unsigned int partial_length;
 
 total_chgs = 0;
-partial_length = 0;
+/* partial_length = 0; */ /* scan-build : never read */
 
 iter_hi = iter + partial_iter;
 if(iter_hi > lt->numfacs) iter_hi = lt->numfacs;
@@ -1491,8 +1491,6 @@ if(lt)
 
 	if( (!lt->timepos) && (!lt->timegranule) && ((!lt->numblock)||(!lt->no_checkpoint)) )
 	        {
-	        struct lxt2_wr_symbol *s = lt->symchain;
-
 		/* fprintf(stderr, "initial value burst timepos==0, timegranule==0\n"); */
 		if(lt->blackout)
 			{
@@ -1501,7 +1499,7 @@ if(lt)
 			}
 		else
 			{
-			s = lt->symchain;
+			struct lxt2_wr_symbol *s = lt->symchain;
 		        while(s)
 		                {
 		                if((!(s->flags&LXT2_WR_SYM_F_ALIAS))&&(s->rows<2))
@@ -2150,8 +2148,8 @@ if(lt)
 		
 		while(s)
 			{
-			if(s->name) { free(s->name); }
-			if(s->value) { free(s->value); }
+			free(s->name);
+			free(s->value);
 			s2=s->symchain;
 			free(s);
 			s=s2;
@@ -2180,17 +2178,3 @@ if(lt)
 	}
 }
 
-/*
- * $Id: lxt2_write.c,v 1.2 2008/12/20 05:08:26 gtkwave Exp $
- * $Log: lxt2_write.c,v $
- * Revision 1.2  2008/12/20 05:08:26  gtkwave
- * -Wshadow warning cleanups
- *
- * Revision 1.1.1.1  2007/05/30 04:28:21  gtkwave
- * Imported sources
- *
- * Revision 1.2  2007/04/20 02:08:18  gtkwave
- * initial release
- *
- */
-
