@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2010 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2011 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -50,7 +50,7 @@ static PLI_INT32 my_mcd_printf(PLI_UINT32 mcd, const char *fmt, ...)
 struct timeformat_info_s timeformat_info = { 0, 0, 0, 20 };
 
 struct strobe_cb_info {
-      char*name;
+      const char*name;
       char*filename;
       int lineno;
       int default_format;
@@ -129,7 +129,7 @@ static void array_from_iterator(struct strobe_cb_info*info, vpiHandle argv)
       }
 }
 
-static int get_default_format(char *name)
+static int get_default_format(const char *name)
 {
     int default_format;
 
@@ -171,7 +171,7 @@ static char * format_as_string(int ljust, int plus, int ld_zero, int width,
     size += sprintf(&buf[size], "%d", width);
   if (prec != -1)
     size += sprintf(&buf[size], ".%d", prec);
-  buf[size++] = fmt;
+  if (fmt) buf[size++] = fmt;
   /* The same goes here ">"! */
   buf[size++] = '>';
   buf[size] = '\0';
@@ -247,16 +247,18 @@ static void get_time(char *rtn, const char *value, int prec,
 static void get_time_real(char *rtn, double value, int prec,
                           PLI_INT32 time_units)
 {
-  /* Scale the value if its time units differ from the format units. */
-  if (time_units != timeformat_info.units) {
+  /* Scale the value from its time units to the format time units. */
+  if (time_units >= timeformat_info.units) {
     value *= pow(10.0, time_units - timeformat_info.units);
+  } else {
+    value /= pow(10.0, timeformat_info.units - time_units);
   }
   sprintf(rtn, "%0.*f%s", prec, value, timeformat_info.suff);
 }
 
 static unsigned int get_format_char(char **rtn, int ljust, int plus,
                                     int ld_zero, int width, int prec,
-                                    char fmt, struct strobe_cb_info *info,
+                                    char fmt, const struct strobe_cb_info *info,
                                     unsigned int *idx)
 {
   s_vpi_value value;
@@ -276,6 +278,7 @@ static unsigned int get_format_char(char **rtn, int ljust, int plus,
   switch (fmt) {
 
     case '%':
+    case '\0':
       if (ljust != 0  || plus != 0 || ld_zero != 0 || width != -1 ||
           prec != -1) {
         vpi_printf("WARNING: %s:%d: invalid format %s%s.\n",
@@ -442,7 +445,9 @@ static unsigned int get_format_char(char **rtn, int ljust, int plus,
           strcpy(cpb+pad, cp);
 
           /* If a width was not given, use the default, unless we have a
-           * leading zero (width of zero). */
+           * leading zero (width of zero). Because the width of a real in
+           * Icarus is 1 the string length will set the width of a real
+           * displayed using %d. */
           if (width == -1) {
             width = (ld_zero == 1) ? 0 : vpi_get_dec_size(info->items[*idx]);
           }
@@ -606,7 +611,7 @@ static unsigned int get_format_char(char **rtn, int ljust, int plus,
           /* The 512 (513-1 for EOL) is more than enough for any double
            * value (309 digits plus a decimal point maximum). Because of
            * scaling this could be larger. For decimal values you can
-           * have an arbitraty value so you can overflow the buffer, but
+           * have an arbitrary value so you can overflow the buffer, but
            * for now we will assume the user will use this as intended
            * (pass a time variable or the result of a time function). */
           tbuf = malloc((513+suff_len)*sizeof(char));
@@ -804,7 +809,7 @@ static unsigned int get_format_char(char **rtn, int ljust, int plus,
 /* We can't use the normal str functions on the return value since
  * %u and %z can insert NULL characters into the stream. */
 static unsigned int get_format(char **rtn, char *fmt,
-                               struct strobe_cb_info *info, unsigned int *idx)
+                               const struct strobe_cb_info *info, unsigned int *idx)
 {
   char *cp = fmt;
   unsigned int size;
@@ -844,17 +849,17 @@ static unsigned int get_format(char **rtn, char *fmt,
       memcpy(*rtn+size-1, result, cnt);
       free(result);
       size += cnt;
-      cp += 1;
+      if (*cp) cp += 1;
     }
   }
   *(*rtn+size-1) = '\0';
   return size - 1;
 }
 
-static unsigned int get_numeric(char **rtn, struct strobe_cb_info *info,
+static unsigned int get_numeric(char **rtn, const struct strobe_cb_info *info,
                                 vpiHandle item)
 {
-  int size;
+  int size, min;
   s_vpi_value val;
 
   val.format = info->default_format;
@@ -863,6 +868,11 @@ static unsigned int get_numeric(char **rtn, struct strobe_cb_info *info,
   switch(info->default_format){
     case vpiDecStrVal:
       size = vpi_get_dec_size(item);
+	/* -1 can be represented as a one bit signed value. This returns	
+	 * a size of 1 which is too small for the -1 string value so make
+	 * the string width the minimum display width. */
+      min = strlen(val.value.str);
+      if (size < min) size = min;
       *rtn = malloc((size+1)*sizeof(char));
       sprintf(*rtn, "%*s", size, val.value.str);
       break;
@@ -875,9 +885,10 @@ static unsigned int get_numeric(char **rtn, struct strobe_cb_info *info,
 
 /* In many places we can't use the normal str functions since %u and %z
  * can insert NULL characters into the stream. */
-static char *get_display(unsigned int *rtnsz, struct strobe_cb_info *info)
+static char *get_display(unsigned int *rtnsz, const struct strobe_cb_info *info)
 {
   char *result, *fmt, *rtn, *func_name;
+  const char *cresult;
   s_vpi_value value;
   unsigned int idx, size, width;
   char buf[256];
@@ -998,10 +1009,10 @@ static char *get_display(unsigned int *rtnsz, struct strobe_cb_info *info)
         vpi_printf("WARNING: %s:%d: unknown argument type (%s) given to %s!\n",
                    info->filename, info->lineno, vpi_get_str(vpiType, item),
                    info->name);
-        result = "<?>";
-        width = strlen(result);
+        cresult = "<?>";
+        width = strlen(cresult);
         rtn = realloc(rtn, (size+width)*sizeof(char));
-        memcpy(rtn+size-1, result, width);
+        memcpy(rtn+size-1, cresult, width);
         break;
     }
     size += width;
@@ -1011,7 +1022,7 @@ static char *get_display(unsigned int *rtnsz, struct strobe_cb_info *info)
   return rtn;
 }
 
-static int sys_check_args(vpiHandle callh, vpiHandle argv, PLI_BYTE8*name,
+static int sys_check_args(vpiHandle callh, vpiHandle argv, const PLI_BYTE8*name,
                           int no_auto, int is_monitor)
 {
       vpiHandle arg;
@@ -1188,8 +1199,6 @@ static PLI_INT32 sys_display_calltf(PLI_BYTE8 *name)
  */
 static PLI_INT32 strobe_cb(p_cb_data cb)
 {
-      char* result = NULL;
-      unsigned int size, location=0;
       struct strobe_cb_info*info = (struct strobe_cb_info*)cb->user_data;
 
 	/* We really need to cancel any $fstrobe() calls for a file when it
@@ -1197,6 +1206,8 @@ static PLI_INT32 strobe_cb(p_cb_data cb)
 	 * Which has the same basic effect. */
       if ((! IS_MCD(info->fd_mcd) && vpi_get_file(info->fd_mcd) != NULL) ||
           ( IS_MCD(info->fd_mcd) && my_mcd_printf(info->fd_mcd, "") != EOF)) {
+	    char* result = NULL;
+	    unsigned int size, location=0;
 	      /* Because %u and %z may put embedded NULL characters into the
 	       * returned string strlen() may not match the real size! */
 	    result = get_display(&size, info);
@@ -1210,12 +1221,12 @@ static PLI_INT32 strobe_cb(p_cb_data cb)
 		  }
 	    }
 	    my_mcd_printf(info->fd_mcd, "\n");
+	    free(result);
       }
 
       free(info->filename);
       free(info->items);
       free(info);
-      free(result);
       return 0;
 }
 
@@ -1755,9 +1766,9 @@ static PLI_INT32 sys_timeformat_calltf(PLI_BYTE8*xx)
       return 0;
 }
 
-static char *pts_convert(int value)
+static const char *pts_convert(int value)
 {
-      char *string;
+      const char *string;
       switch (value) {
             case   0: string = "1s";    break;
             case  -1: string = "100ms"; break;
