@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2010 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2008-2011 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -407,7 +407,7 @@ int vpip_time_precision_from_handle(vpiHandle obj)
 
 void vpi_get_time(vpiHandle obj, s_vpi_time*vp)
 {
-      int units;
+      int scale;
       vvp_time64_t time;
 
       assert(vp);
@@ -421,9 +421,10 @@ void vpi_get_time(vpiHandle obj, s_vpi_time*vp)
 	    break;
 
           case vpiScaledRealTime:
-	    units = vpip_time_units_from_handle(obj);
-            vp->real = pow(10.0L, vpip_get_time_precision() - units);
-            vp->real *= time;
+	    scale = vpip_get_time_precision() -
+	            vpip_time_units_from_handle(obj);
+	    if (scale >= 0) vp->real = (double)time * pow(10.0, scale);
+	    else vp->real = (double)time / pow(10.0, -scale);
 	    break;
 
           default:
@@ -840,6 +841,7 @@ vpiHandle vpi_put_value(vpiHandle obj, s_vpi_value*vp,
 
       if (flags!=vpiNoDelay && flags!=vpiForceFlag && flags!=vpiReleaseFlag) {
 	    vvp_time64_t dly;
+	    int scale;
 
             if (vpi_get(vpiAutomatic, obj)) {
                   fprintf(stderr, "vpi error: cannot put a value with "
@@ -853,10 +855,13 @@ vpiHandle vpi_put_value(vpiHandle obj, s_vpi_value*vp,
 
 	    switch (when->type) {
 		case vpiScaledRealTime:
-		  dly = (vvp_time64_t)(when->real *
-				       (pow(10.0L,
-					    vpip_time_units_from_handle(obj) -
-					    vpip_get_time_precision())));
+		  scale = vpip_time_units_from_handle(obj) -
+		          vpip_get_time_precision();
+		  if (scale >= 0) {
+			dly = (vvp_time64_t)(when->real * pow(10.0, scale));
+		  } else {
+			dly = (vvp_time64_t)(when->real / pow(10.0, -scale));
+		  }
 		  break;
 		case vpiSimTime:
 		  dly = vpip_timestruct_to_time(when);
