@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2010 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -30,16 +30,13 @@
 #ifdef CHECK_WITH_VALGRIND
 # include  "vvp_cleanup.h"
 #endif
-# include  <math.h>
+# include  <cmath>
 # include  <iostream>
-# include  <stdio.h>
-#ifdef HAVE_MALLOC_H
-# include  <malloc.h>
-#endif
-# include  <stdlib.h>
-# include  <limits.h>
-# include  <string.h>
-# include  <assert.h>
+# include  <cstdio>
+# include  <cstdlib>
+# include  <climits>
+# include  <cstring>
+# include  <cassert>
 #ifdef CHECK_WITH_VALGRIND
 # include  <valgrind/memcheck.h>
 #endif
@@ -95,38 +92,41 @@ char *need_result_buf(unsigned cnt, vpi_rbuf_t type)
 
 char *simple_set_rbuf_str(const char *s1)
 {
-	char *res = need_result_buf(strlen(s1)+1, RBUF_STR);
-	if (res) strcpy(res,s1);
-	return res;
+      char *res = need_result_buf(strlen(s1)+1, RBUF_STR);
+      if (res) strcpy(res,s1);
+      return res;
 }
 
 char *generic_get_str(int code, vpiHandle ref, const char *name, const char *index)
 {
-	size_t len = strlen(name) + 1;  /* include space for null termination */
-	char *bn = NULL;
-	if (code == vpiFullName) {
-		bn = strdup(vpi_get_str(code,ref));
-		len += strlen(bn) + 1;  /* include space for "." separator */
-	}
-	if (index != NULL) len += strlen(index) + 2;  /* include space for brackets */
+      size_t len = strlen(name) + 1;  /* include space for null termination */
+      char *bn = NULL;
+      if (code == vpiFullName) {
+	    bn = strdup(vpi_get_str(code,ref));
+	    len += strlen(bn) + 1;  /* include space for "." separator */
+      }
+      if (index != NULL) len += strlen(index) + 2;  /* include space for brackets */
 
-	char *res = need_result_buf(len, RBUF_STR);
-	if (!res) return NULL;
-	*res=0;  /* start with nothing */
+      char *res = need_result_buf(len, RBUF_STR);
+      if (!res) {
+	    free(bn);
+	    return NULL;
+      }
+      *res=0;  /* start with nothing */
 
 	/* if this works, I can make it more efficient later */
-	if (bn != NULL) {
-		strcat(res, bn);
-		strcat(res, ".");
-		free(bn);
-	}
-	strcat(res, name);
-	if (index != NULL) {
-		strcat(res, "[");
-		strcat(res, index);
-		strcat(res, "]");
-	}
-	return res;
+      if (bn != NULL) {
+	    strcat(res, bn);
+	    strcat(res, ".");
+	    free(bn);
+      }
+      strcat(res, name);
+      if (index != NULL) {
+	    strcat(res, "[");
+	    strcat(res, index);
+	    strcat(res, "]");
+      }
+      return res;
 }
 
 /*
@@ -1105,14 +1105,19 @@ static char* PV_get_str(int code, vpiHandle ref)
 	case vpiName:
 	case vpiFullName: {
 	    const char*nm = vpi_get_str(code, rfp->parent);
-	    char full[1024+strlen(nm)];
-	    sprintf(full, "%s[%d:%d]", nm, (int)vpi_get(vpiLeftRange, ref),
-	                                   (int)vpi_get(vpiRightRange, ref));
-	    return simple_set_rbuf_str(full);
+	    size_t len = 256+strlen(nm);
+	    char *full = (char *) malloc(len);
+	    snprintf(full, len, "%s[%d:%d]", nm,
+	                                     (int)vpi_get(vpiLeftRange, ref),
+	                                     (int)vpi_get(vpiRightRange, ref));
+	    full[len-1] = 0;
+	    char *res = simple_set_rbuf_str(full);
+	    free(full);
+	    return res;
 	}
 
 	default:
-	    fprintf(stderr, "PV_get_str: property %d is unknown\n", code);
+	    fprintf(stderr, "PV_get_str: property %d is unknown.\n", code);
       }
 
       return 0;
